package com.nju.svcdisambiguation.serviceImp;

import com.nju.svcdisambiguation.dataMapper.acm.ACMAuthorDetailMapper;
import com.nju.svcdisambiguation.dataMapper.ieee.IEEEAuthorDetailMapper;
import com.nju.svcdisambiguation.po.FieldSummaryPO;
import com.nju.svcdisambiguation.po.PaperActivationPO;
import com.nju.svcdisambiguation.service.AuthorDetailService;
import com.nju.svcdisambiguation.vo.ResponseVO;
import com.nju.svcdisambiguation.vo.detail.author.AuthorDetailVO;
import com.nju.svcdisambiguation.vo.detail.author.AuthorFieldSummaryVO;
import com.nju.svcdisambiguation.vo.detail.author.InstitutionAuthorDetailVO;
import com.nju.svcdisambiguation.vo.detail.author.KeywordsAuthorDetailVO;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Calendar;

@Service
public class AuthorDetailServiceImp implements AuthorDetailService {

    private final IEEEAuthorDetailMapper ieeeAuthorDetailMapper;
    private final ACMAuthorDetailMapper acmAuthorDetailMapper;

    public AuthorDetailServiceImp(IEEEAuthorDetailMapper ieeeAuthorDetailMapper, ACMAuthorDetailMapper acmAuthorDetailMapper) {
        this.ieeeAuthorDetailMapper = ieeeAuthorDetailMapper;
        this.acmAuthorDetailMapper = acmAuthorDetailMapper;
    }

    @Override
    public ResponseVO getAuthorDetail(int authorId, String db) {
        if (db.equals("ACM")) {
            AuthorDetailVO authorDetailVO = acmAuthorDetailMapper.selectAuthorDetailById(authorId);
            ArrayList<InstitutionAuthorDetailVO> institutionAuthorDetailVOS = acmAuthorDetailMapper.selectInstitutionByAuthorId(authorId);
            for (InstitutionAuthorDetailVO institutionAuthorDetailVO:institutionAuthorDetailVOS){
                institutionAuthorDetailVO.setDb("ACM");
            }
            authorDetailVO.setInstitution(institutionAuthorDetailVOS);
            ArrayList<PaperActivationPO> paperActivationPOS = acmAuthorDetailMapper.selectPaperActivationPOByAuthorId(authorId);
            authorDetailVO.setActivation(calculateActivation(paperActivationPOS));
            authorDetailVO.setTotalCitations(calculateTotalCitations(paperActivationPOS));
            authorDetailVO.setSameAuthors(acmAuthorDetailMapper.selectSameAuthorsByAuthorId(authorId));
            authorDetailVO.setDocCount(acmAuthorDetailMapper.selectDocCountByAuthorId(authorId));
            authorDetailVO.setFieldList(acmAuthorDetailMapper.selectKeywordsAuthorDetailByAuthorId(authorId));
            for(KeywordsAuthorDetailVO keywordsAuthorDetailVO:authorDetailVO.getFieldList()){
                keywordsAuthorDetailVO.setActivation(calculateActivation(acmAuthorDetailMapper.selectPAVOByFieldNameAndAuthorId(
                        keywordsAuthorDetailVO.getName(),
                        authorId
                )));
            }
            return ResponseVO.buildSuccess(authorDetailVO);
        }

        AuthorDetailVO authorDetailVO = ieeeAuthorDetailMapper.selectAuthorDetailById(authorId);
        ArrayList<InstitutionAuthorDetailVO> institutionAuthorDetailVOS = ieeeAuthorDetailMapper.selectInstitutionByAuthorId(authorId);
        for (InstitutionAuthorDetailVO institutionAuthorDetailVO:institutionAuthorDetailVOS){
            institutionAuthorDetailVO.setDb("IEEE");
        }
        authorDetailVO.setInstitution(institutionAuthorDetailVOS);
        ArrayList<PaperActivationPO> paperActivationPOS = ieeeAuthorDetailMapper.selectPaperActivationPOByAuthorId(authorId);
        authorDetailVO.setActivation(calculateActivation(paperActivationPOS));
        authorDetailVO.setTotalCitations(calculateTotalCitations(paperActivationPOS));
        authorDetailVO.setSameAuthors(ieeeAuthorDetailMapper.selectSameAuthorsByAuthorId(authorId));
        authorDetailVO.setDocCount(ieeeAuthorDetailMapper.selectDocCountByAuthorId(authorId));
        authorDetailVO.setFieldList(ieeeAuthorDetailMapper.selectKeywordsAuthorDetailByAuthorId(authorId));
        for(KeywordsAuthorDetailVO keywordsAuthorDetailVO:authorDetailVO.getFieldList()){
            keywordsAuthorDetailVO.setActivation(calculateActivation(acmAuthorDetailMapper.selectPAVOByFieldNameAndAuthorId(
                    keywordsAuthorDetailVO.getName(),
                    authorId
            )));
        }
        return ResponseVO.buildSuccess(authorDetailVO);
    }

    private int calculateTotalCitations(ArrayList<PaperActivationPO> paperActivationPOS) {
        int ret = 0;
        for (PaperActivationPO paperActivationPO:paperActivationPOS){
            ret += paperActivationPO.getCitations();
        }
        return ret;
    }

    private double calculateActivation(ArrayList<PaperActivationPO> paperActivationPOS) {
        double ret = 0.0;
        Calendar date = Calendar.getInstance();
        int year = date.get(Calendar.YEAR);

        for(PaperActivationPO paperActivationPO:paperActivationPOS){
            ret += (double)paperActivationPO.getCitations()/(year - paperActivationPO.getYear() + 1.0);
        }

        return ret;
    }

    @Override
    public ResponseVO getAuthorPublicationSummary(int authorId, String db) {
        if (db.equals("ACM")) {
            return ResponseVO.buildSuccess(acmAuthorDetailMapper.selectAuthorPublicationSummaryByAuthorId(authorId));
        }
        return ResponseVO.buildSuccess(ieeeAuthorDetailMapper.selectAuthorPublicationSummaryByAuthorId(authorId));
    }

    @Override
    public ResponseVO getAuthorFieldSummary(int authorId, String db) {
        ArrayList<FieldSummaryPO> fieldSummaryPOS = new ArrayList<>();
        ArrayList<AuthorFieldSummaryVO> authorFieldSummaryVOS = new ArrayList<>();
        if (db.equals("ACM")){
            fieldSummaryPOS = acmAuthorDetailMapper.selectFieldSummaryPOByAuthorId(authorId);
            for (FieldSummaryPO fieldSummaryPO:fieldSummaryPOS){
                AuthorFieldSummaryVO authorFieldSummaryVO = new AuthorFieldSummaryVO(
                        fieldSummaryPO.getFieldId(),
                        fieldSummaryPO.getFieldName(),
                        fieldSummaryPO.getCount(),
                        "ACM"
                );
                authorFieldSummaryVOS.add(authorFieldSummaryVO);
            }
            return ResponseVO.buildSuccess(authorFieldSummaryVOS);
        }
        fieldSummaryPOS = ieeeAuthorDetailMapper.selectFieldSummaryPOByAuthorId(authorId);
        for (FieldSummaryPO fieldSummaryPO:fieldSummaryPOS){
            AuthorFieldSummaryVO authorFieldSummaryVO = new AuthorFieldSummaryVO(
                    fieldSummaryPO.getFieldId(),
                    fieldSummaryPO.getFieldName(),
                    fieldSummaryPO.getCount(),
                    "IEEE"
            );
            authorFieldSummaryVOS.add(authorFieldSummaryVO);
        }
        return ResponseVO.buildSuccess(authorFieldSummaryVOS);
    }

}
